package artcustomer.limon.helloworld.api {
	import flash.display.DisplayObjectContainer;
	
	import artcustomer.framework.context.FlashContext;
	import artcustomer.framework.events.*;
	import artcustomer.framework.utils.consts.*;
	import artcustomer.framework.debug.console.ConsoleLog;
	
	import artcustomer.limon.helloworld.components.application.*;
	import artcustomer.limon.helloworld.components.application.model.*;
	import artcustomer.limon.helloworld.components.application.command.*;
	
	
	/**
	 * LimonHelloWorld
	 * 
	 * @author David Massenot
	 */
	public class LimonHelloWorld extends FlashContext {
		
		
		/**
		 * Constructor
		 */
		public function LimonHelloWorld() {
			super();
			
			this.name = 'Limon Hello World';
			this.mode = ContextMode.DEV;
		}
		
		//---------------------------------------------------------------------
		//  Events
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function listenEvents():void {
			this.addEventListener(ContextErrorEvent.ERROR, handleContextError, false, 0, true);
			this.addEventListener(ContextErrorEvent.FRAMEWORK_ERROR, handleContextError, false, 0, true);
			this.addEventListener(ContextErrorEvent.ILLEGAL_ERROR, handleContextError, false, 0, true);
			this.addEventListener(ContextEvent.CONTEXT_SETUP, handleContext, false, 0, true);
			this.addEventListener(ContextEvent.CONTEXT_DESTROY, handleContext, false, 0, true);
		}
		
		/**
		 * @private
		 */
		private function unlistenEvents():void {
			this.removeEventListener(ContextErrorEvent.ERROR, handleContextError);
			this.removeEventListener(ContextErrorEvent.FRAMEWORK_ERROR, handleContextError);
			this.removeEventListener(ContextErrorEvent.ILLEGAL_ERROR, handleContextError);
			this.removeEventListener(ContextEvent.CONTEXT_SETUP, handleContext);
			this.removeEventListener(ContextEvent.CONTEXT_DESTROY, handleContext);
		}
		
		//---------------------------------------------------------------------
		//  Listeners
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function handleContextError(e:ContextErrorEvent):void {
			var allowError:Boolean = true;
			var errorName:String = e.errorName +  ' ' + e.error.errorID;
			var errorText:String = '';
			
			switch (this.mode) {
				case(ContextMode.RELEASE):
					allowError = true;
					errorText = String(e.error.message);
					break;
					
				case(ContextMode.DEV):
					allowError = true;
					errorText = String(e.error.message);
					break;
					
				case(ContextMode.DEBUG):
					allowError = true;
					errorText = String(e.error.getStackTrace());
					break;
					
				default:
					allowError = false;
					break;
			}
			
			if (allowError) ConsoleLog.log(errorName + ' : ' + errorText, ConsoleLog.LEVEL_ERROR);
			
			e.stopPropagation();
		}
		
		/**
		 * @private
		 */
		private function handleContext(e:ContextEvent):void {
			switch (e.type) {
				case(ContextEvent.CONTEXT_SETUP):
					ConsoleLog.log('Application is started !', ConsoleLog.LEVEL_INFOS);
					break;
					
				case(ContextEvent.CONTEXT_DESTROY):
					ConsoleLog.log('Application is terminated !', ConsoleLog.LEVEL_INFOS);
					break;
				
				default:
					break;
			}
		}
		
		//---------------------------------------------------------------------
		//  Components 
		//---------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private function addDefaultComponents():void {
			this.addComponent(CompApplication, ApplicationModel, ApplicationCommand);
		}
		
		
		/**
		 * Setup Application. Entry point.
		 */
		override public function setup():void {
			listenEvents();
			
			super.setup();
			
			this.moveLogo(LogoPosition.BOTTOM_RIGHT, 5, 10);
			
			addDefaultComponents();
		}
		
		/**
		 * Destroy Application. Exit point.
		 */
		override public function destroy():void {
			unlistenEvents();
			
			super.destroy();
		}
	}
}